@@ -1071,4 +1071,60 @@ setSelectedOutputStringOn(SEXP nuser, SEXP value)
 }
 
 
+#include <R_ext/Rdynload.h>
+
+#define CALLDEF(name, n)  {#name, (DL_FUNC) &name, n}
+
+const static R_CallMethodDef R_CallDef[] = {
+  CALLDEF(accumLineLst, 1),
+  CALLDEF(clearAccum, 0),
+  CALLDEF(getAccumLines, 0),
+  CALLDEF(listComps, 0),
+  CALLDEF(getDumpFileName, 0),
+  CALLDEF(getDumpStrings, 0),
+  CALLDEF(getErrorFileName, 0),
+  CALLDEF(getDumpFileOn, 0),
+  CALLDEF(getDumpStringOn, 0),
+  CALLDEF(getErrorFileOn, 0),
+  CALLDEF(getErrorStringOn, 0),
+  CALLDEF(getLogFileOn, 0),
+  CALLDEF(getLogStringOn, 0),
+  CALLDEF(getOutputFileOn, 0),
+  CALLDEF(getOutputStringOn, 0),
+  CALLDEF(getErrorStrings, 0),
+  CALLDEF(getLogFileName, 0),
+  CALLDEF(getLogStrings, 0),
+  CALLDEF(getOutputFileName, 0),
+  CALLDEF(getOutputStrings, 0),
+  CALLDEF(getSelOutLst, 0),
+  CALLDEF(getWarningStrings, 0),
+  CALLDEF(loadDB, 1),
+  CALLDEF(loadDBLst, 1),
+  CALLDEF(runAccum, 0),
+  CALLDEF(runFile, 1),
+  CALLDEF(runStringLst, 1),
+  CALLDEF(setDumpFileName, 1),
+  CALLDEF(setDumpFileOn, 1),
+  CALLDEF(setDumpStringOn, 1),
+  CALLDEF(setErrorFileName, 1),
+  CALLDEF(setErrorFileOn, 1),
+  CALLDEF(setErrorStringOn, 1),
+  CALLDEF(setLogFileName, 1),
+  CALLDEF(setLogFileOn, 1),
+  CALLDEF(setLogStringOn, 1),
+  CALLDEF(setOutputFileName, 1),
+  CALLDEF(setOutputFileOn, 1),
+  CALLDEF(setOutputStringOn, 1),
+  CALLDEF(getSelectedOutputFileName, 1),
+  CALLDEF(setSelectedOutputFileName, 2),
+  CALLDEF(setSelectedOutputFileOn, 2),
+  {NULL, NULL, 0}
+};
+
+void R_init_phreeqc(DllInfo *dll)
+{
+  R_registerRoutines(dll, NULL, R_CallDef, NULL, NULL);
+  R_useDynamicSymbols(dll, FALSE);
+}
+
 } // extern "C"
