@@ -3,8 +3,14 @@
 
 #include <sstream>
 
+#if defined(_WINDLL)
+#define IPQ_DLL_EXPORT __declspec(dllexport)
+#else
+#define IPQ_DLL_EXPORT
+#endif
+
 class PHRQ_io;
-class PHRQ_base
+class IPQ_DLL_EXPORT PHRQ_base
 {
 public:
 	// constructors
